import { ZodType, TypeOf, input, output } from "zod";

/**
 * NonEmptyArray
 */
interface NonEmptyArray<T> extends Array<T> {
  0: T;
}
declare const of$1: <T>(value: T) => NonEmptyArray<T>;
declare function isNonEmpty<T>(as: Array<T>): as is NonEmptyArray<T>;
declare function isOutOfBound<T>(index: number, as: Array<T>): boolean;
declare function modifyAt<T>(index: number, fn: (a: T) => T): (as: Array<T>) => Array<T>;
declare function deleteAt<T>(index: number): (as: Array<T>) => Array<T>;

type Predicate<T> = (value: T) => boolean;
type Refinement<A, B extends A> = (value: A) => value is B;

type UnknownRecord = Record<string, unknown>;

type Result<F, S> = Failure<F> | Success<S>;
type Failure<F> = {
  id: "Failure";
  failure: F;
};
declare const failure$1: <F>(failure: F) => Result<F, never>;
declare function isFailure<F, S>(result: Result<F, S>): result is Failure<F>;
type Success<S> = {
  id: "Success";
  success: S;
};
declare const success$1: <S>(success: S) => Result<never, S>;
declare function isSuccess<F, S>(result: Result<F, S>): result is Success<S>;
declare function fromPredicate$1<F, S>(predicate: Predicate<S>, onFailure: () => F): (value: S) => Result<F, S>;
declare function extract<F, S>(result: Result<F, S>): S;
declare function map<F, A, B>(f: (a: A) => B, result: Result<F, A>): Result<F, B>;

declare function useFieldArrayState<T extends UnknownRecord>(
  initialValues: Array<T>,
): {
  state: FormState<T>[];
  errors: FormErrors<T>[];
  values: T[];
  append: (values: T) => void;
  blur: <K extends keyof T>(index: number, key: K) => void;
  change: <K_1 extends keyof T>(index: number, key: K_1, value: T[K_1]) => void;
  disable: <K_2 extends keyof T>(index: number, key: K_2) => void;
  enable: <K_3 extends keyof T>(index: number, key: K_3) => void;
  remove: (index: number) => void;
  reset: (update?: Update<Array<T>>) => void;
  setErrors: SetFieldArrayErrors<T>;
  setValues: (index: number, update: Update<T>) => void;
  propagateErrors: (errors: Array<FormErrors<T> | null>) => void;
  toggle: Toggle;
};

declare function useFormState<T extends UnknownRecord>(
  initialValues: T,
): {
  state: FormState<T>;
  errors: FormErrors<T>;
  values: T;
  blur: <K extends keyof T>(key: K) => void;
  change: <K_1 extends keyof T>(key: K_1, value: T[K_1]) => void;
  disable: <K_2 extends keyof T>(key: K_2) => void;
  enable: <K_3 extends keyof T>(key: K_3) => void;
  reset: (update?: Update<T>) => void;
  setErrors: SetErrors<T>;
  setValues: (update: Update<T>) => void;
  propagateErrors: (errors: FormErrors<T>) => void;
  toggle: Toggle;
};

type Validator<I, O> = (input: I) => Promise<Result<NonEmptyArray<string>, O>>;
declare const of: <T>(value: T) => ReturnType<Validator<T, T>>;
declare const success: <T>(value: T) => ReturnType<Validator<T, T>>;
declare function failure<T>(errors: NonEmptyArray<string>): ReturnType<Validator<T, T>>;
declare function fromPredicate<A, B extends A>(predicate: Refinement<A, B>, message: string): Validator<A, B>;
declare function fromPredicate<A>(predicate: Predicate<A>, message: string): Validator<A, A>;
declare function tryCatch<I, A, O>(
  action: (input: I) => Promise<A>,
  {
    onFailure,
    onSuccess,
  }: {
    onFailure: (value: unknown) => string;
    onSuccess: (value: A) => O;
  },
): Validator<I, O>;
declare function transform<I, A, B>(f: (a: A) => B): (validator: Validator<I, A>) => Validator<I, B>;
declare function transform<I, A, B>(f: (a: A) => B, validator: Validator<I, A>): Validator<I, B>;
declare function chain<I, A, O>(ao: Validator<A, O>): (ia: Validator<I, A>) => Validator<I, O>;
declare function orElse<I, O>(first: Validator<I, O>): (second: Validator<I, O>) => Validator<I, O>;
declare function orElse<I, O>(first: Validator<I, O>, second: Validator<I, O>): Validator<I, O>;
/**
 * Sequence
 *
 * Fast-failing validation - returns a `NonEmptyArray<string>` once some
 * validation fails
 */
declare function sequence<I, A, O>(a: Validator<I, A>, b: Validator<A, O>): Validator<I, O>;
declare function sequence<I, A, B, O>(a: Validator<I, A>, b: Validator<A, B>, c: Validator<B, O>): Validator<I, O>;
declare function sequence<I, A, B, C, O>(
  a: Validator<I, A>,
  b: Validator<A, B>,
  c: Validator<B, C>,
  d: Validator<C, O>,
): Validator<I, O>;
declare function sequence<I, A, B, C, D, O>(
  a: Validator<I, A>,
  b: Validator<A, B>,
  c: Validator<B, C>,
  d: Validator<C, D>,
  e: Validator<D, O>,
): Validator<I, O>;
declare function sequence<I, A, B, C, D, E, O>(
  a: Validator<I, A>,
  b: Validator<A, B>,
  c: Validator<B, C>,
  d: Validator<C, D>,
  e: Validator<D, E>,
  f: Validator<E, O>,
): Validator<I, O>;
declare function sequence<I, A, B, C, D, E, F, O>(
  a: Validator<I, A>,
  b: Validator<A, B>,
  c: Validator<B, C>,
  d: Validator<C, D>,
  e: Validator<D, E>,
  f: Validator<E, F>,
  g: Validator<F, O>,
): Validator<I, O>;
declare function sequence<I, A, B, C, D, E, F, G, O>(
  a: Validator<I, A>,
  b: Validator<A, B>,
  c: Validator<B, C>,
  d: Validator<C, D>,
  e: Validator<D, E>,
  f: Validator<E, F>,
  g: Validator<F, G>,
  h: Validator<G, O>,
): Validator<I, O>;
declare function sequence<I, A, B, C, D, E, F, G, H, O>(
  a: Validator<I, A>,
  b: Validator<A, B>,
  c: Validator<B, C>,
  d: Validator<C, D>,
  e: Validator<D, E>,
  f: Validator<E, F>,
  g: Validator<F, G>,
  h: Validator<G, H>,
  i: Validator<H, I>,
): Validator<I, O>;
/**
 * Parallel
 *
 * Applies validators in parallel and collects all validation errors
 */
declare function parallel<I, O>(a: Validator<I, O>, b: Validator<I, O>): Validator<I, O>;
declare function parallel<I, O>(a: Validator<I, O>, b: Validator<I, O>, c: Validator<I, O>): Validator<I, O>;
declare function parallel<I, O>(
  a: Validator<I, O>,
  b: Validator<I, O>,
  c: Validator<I, O>,
  d: Validator<I, O>,
): Validator<I, O>;
declare function parallel<I, O>(
  a: Validator<I, O>,
  b: Validator<I, O>,
  c: Validator<I, O>,
  d: Validator<I, O>,
  e: Validator<I, O>,
): Validator<I, O>;
declare function parallel<I, O>(
  a: Validator<I, O>,
  b: Validator<I, O>,
  c: Validator<I, O>,
  d: Validator<I, O>,
  e: Validator<I, O>,
  f: Validator<I, O>,
): Validator<I, O>;
declare function parallel<I, O>(
  a: Validator<I, O>,
  b: Validator<I, O>,
  c: Validator<I, O>,
  d: Validator<I, O>,
  e: Validator<I, O>,
  f: Validator<I, O>,
  g: Validator<I, O>,
): Validator<I, O>;
declare function parallel<I, O>(
  a: Validator<I, O>,
  b: Validator<I, O>,
  c: Validator<I, O>,
  d: Validator<I, O>,
  e: Validator<I, O>,
  f: Validator<I, O>,
  g: Validator<I, O>,
  h: Validator<I, O>,
): Validator<I, O>;
declare function parallel<I, O>(
  a: Validator<I, O>,
  b: Validator<I, O>,
  c: Validator<I, O>,
  d: Validator<I, O>,
  e: Validator<I, O>,
  f: Validator<I, O>,
  g: Validator<I, O>,
  h: Validator<I, O>,
  i: Validator<I, O>,
): Validator<I, O>;
declare function defined<A>(message: string): Validator<A, NonNullable<A>>;
/**
 * Basic validators
 */
declare function min<T extends number>(min: number, message: string): Validator<T, T>;
declare function max<T extends number>(max: number, message: string): Validator<T, T>;
declare function range<T extends number>(min: number, max: number, message: string): Validator<T, T>;
declare function minLength<T extends string>(min: number, message: string): Validator<T, T>;
declare function maxLength<T extends string>(max: number, message: string): Validator<T, T>;
declare function lengthRange<T extends string>(min: number, max: number, message: string): Validator<T, T>;
declare function match<T extends string>(pattern: RegExp, message: string): Validator<T, T>;

type validator_Validator<I, O> = Validator<I, O>;
declare const validator_chain: typeof chain;
declare const validator_defined: typeof defined;
declare const validator_failure: typeof failure;
declare const validator_fromPredicate: typeof fromPredicate;
declare const validator_lengthRange: typeof lengthRange;
declare const validator_match: typeof match;
declare const validator_max: typeof max;
declare const validator_maxLength: typeof maxLength;
declare const validator_min: typeof min;
declare const validator_minLength: typeof minLength;
declare const validator_of: typeof of;
declare const validator_orElse: typeof orElse;
declare const validator_parallel: typeof parallel;
declare const validator_range: typeof range;
declare const validator_sequence: typeof sequence;
declare const validator_success: typeof success;
declare const validator_transform: typeof transform;
declare const validator_tryCatch: typeof tryCatch;
declare namespace validator {
  export {
    type validator_Validator as Validator,
    validator_chain as chain,
    validator_defined as defined,
    validator_failure as failure,
    validator_fromPredicate as fromPredicate,
    validator_lengthRange as lengthRange,
    validator_match as match,
    validator_max as max,
    validator_maxLength as maxLength,
    validator_min as min,
    validator_minLength as minLength,
    validator_of as of,
    validator_orElse as orElse,
    validator_parallel as parallel,
    validator_range as range,
    validator_sequence as sequence,
    validator_success as success,
    validator_transform as transform,
    validator_tryCatch as tryCatch,
  };
}

type UnknownFieldArray = Record<string, Array<UnknownRecord>>;
type FieldState<T> = {
  disabled: boolean;
  errors: NonEmptyArray<string> | null;
  touched: boolean;
  value: T;
};
type FieldProps<T> = FieldState<T> & {
  onBlur: () => void;
  onChange: (value: T) => void;
};
type FormState<T extends UnknownRecord> = {
  [K in keyof T]: FieldState<T[K]>;
};
type FieldGroup<T extends UnknownRecord> = {
  [K in keyof T]: FieldProps<T[K]>;
};
type FormErrors<T extends UnknownRecord> = {
  [K in keyof T]: FieldState<T>["errors"];
};
type SetErrors<T extends UnknownRecord> = {
  (key: keyof T, errors: NonEmptyArray<string> | null): void;
};
type SetFieldArrayErrors<T extends UnknownRecord> = {
  (index: number, key: keyof T, errors: NonEmptyArray<string> | null): void;
};
type Update<Values> = (values: Values) => Values;
type Toggle = (action: "enable" | "disable") => void;
type ValidationStrategy = "onChange" | "onBlur" | "onSubmit";
type OnSubmit$1<K extends "Form" | "FieldArray", T extends UnknownRecord, S extends ValidationSchema<T>> = {
  (data: K extends "Form" ? ValidatedValues<T, S> : Array<ValidatedValues<T, S>>): Promise<unknown>;
};
type OnFailure$1<K extends "Form" | "FieldArray", T extends UnknownRecord> = {
  (errors: K extends "Form" ? FormErrors<T> : Array<FormErrors<T> | null>): unknown;
};
type OnSubmitMatch$1<K extends "Form" | "FieldArray", T extends UnknownRecord, S extends ValidationSchema<T>> = {
  onSuccess: OnSubmit$1<K, T, S>;
  onFailure: OnFailure$1<K, T>;
};
type HandleSubmit$1<K extends "Form" | "FieldArray", T extends UnknownRecord, S extends ValidationSchema<T>> = {
  (onSubmit: OnSubmit$1<K, T, S>): void;
  (onSubmit: OnSubmitMatch$1<K, T, S>): void;
};
declare function initializeForm<T extends UnknownRecord>(values: T): FormState<T>;
declare function getValues<T extends UnknownRecord>(formState: FormState<T>): T;
declare function updateValues<T extends UnknownRecord>(formState: FormState<T>, values: T): FormState<T>;
declare function getErrors<T extends UnknownRecord>(formState: FormState<T>): FormErrors<T>;
declare function formStateManager<T extends UnknownRecord>(
  state: FormState<T>,
): {
  blur: <K extends keyof T>(key: K) => FormState<T>;
  change: <K_1 extends keyof T>(key: K_1, value: T[K_1]) => FormState<T>;
  disable: <K_2 extends keyof T>(key: K_2) => FormState<T>;
  enable: <K_3 extends keyof T>(key: K_3) => FormState<T>;
  setErrors: <K_4 extends keyof T>(key: K_4, errors: NonEmptyArray<string> | null) => FormState<T>;
};
type FieldPropsConfig<T extends UnknownRecord, S extends ValidationSchema<T>> = {
  form: ReturnType<typeof useFormState<T>>;
  schema: S;
  validationStrategy: ValidationStrategy;
};
declare function makeFieldProps<T extends UnknownRecord, S extends ValidationSchema<T>>({
  form,
  schema,
  validationStrategy,
}: FieldPropsConfig<T, S>): <K extends keyof T>(key: K) => FieldProps<T[K]>;
type FieldGroupsConfig<T extends UnknownRecord, S extends ValidationSchema<T>> = {
  fieldArray: ReturnType<typeof useFieldArrayState<T>>;
  schema: S;
  validationStrategy: ValidationStrategy;
};
declare function makeFieldGroups<T extends UnknownRecord, S extends ValidationSchema<T>>({
  fieldArray,
  schema,
  validationStrategy,
}: FieldGroupsConfig<T, S>): Array<FieldGroup<T>>;
type ValidationSchema<T extends UnknownRecord> = {
  [K in keyof T]: FieldValidationConfig<T[K], unknown> | Validator<T[K], unknown> | null;
};
type FieldValidationConfig<I, O> = {
  validationStrategy: ValidationStrategy;
  validator: Validator<I, O>;
};
type ValidatedValues<T extends UnknownRecord, S extends ValidationSchema<T>> = {
  [K in keyof T]: S[K] extends FieldValidationConfig<T[K], infer O>
    ? O
    : S[K] extends Validator<T[K], infer O>
      ? O
      : T[K];
};
type ValidationResult<T extends UnknownRecord, S extends ValidationSchema<T>> = Result<
  FormErrors<T>,
  ValidatedValues<T, S>
>;
declare function validateForm<T extends UnknownRecord, S extends ValidationSchema<T>>(
  values: T,
  schema: S,
): Promise<ValidationResult<T, S>>;
type FieldArrayValidationResult<T extends UnknownRecord, S extends ValidationSchema<T>> = Result<
  Array<FormErrors<T> | null>,
  Array<ValidatedValues<T, S>>
>;
declare function validateFieldArray<T extends UnknownRecord, S extends ValidationSchema<T>>(
  values: Array<T>,
  schema: S,
): Promise<FieldArrayValidationResult<T, S>>;
type FieldArrayValidationSchema$1<FieldArrayValues extends UnknownFieldArray> = {
  [K in keyof FieldArrayValues]: ValidationSchema<FieldArrayValues[K][number]>;
};
type FieldArrayValidationFailure<FieldArrayValues extends UnknownFieldArray> = {
  [K in keyof FieldArrayValues]: Array<FormErrors<FieldArrayValues[K][number]> | null> | null;
};
type FieldArrayValidatedValues$1<
  FieldArrayValues extends UnknownFieldArray,
  FieldArraySchema extends FieldArrayValidationSchema$1<FieldArrayValues>,
> = {
  [K in keyof FieldArrayValues]: Array<
    ValidatedValues<
      FieldArrayValues[K][number],
      FieldArraySchema[K] extends ValidationSchema<FieldArrayValues[K][number]>
        ? FieldArraySchema[K]
        : ValidationSchema<FieldArrayValues[K][number]>
    >
  >;
};
type CompositeFieldArrayValidationResult<
  FieldArrayValues extends UnknownFieldArray,
  FieldArraySchema extends FieldArrayValidationSchema$1<FieldArrayValues>,
> = Result<
  FieldArrayValidationFailure<FieldArrayValues>,
  FieldArrayValidatedValues$1<FieldArrayValues, FieldArraySchema>
>;
declare function validateCompositeFieldArray<
  FieldArrayValues extends UnknownFieldArray,
  FieldArraySchema extends FieldArrayValidationSchema$1<FieldArrayValues>,
>(
  values: FieldArrayValues,
  schema: FieldArraySchema,
): Promise<CompositeFieldArrayValidationResult<FieldArrayValues, FieldArraySchema>>;

type UseFieldArrayConfig<T extends UnknownRecord, S extends ValidationSchema<T>> = {
  initialValues: Array<T>;
  validationStrategy: ValidationStrategy;
  validators: (values: Array<T>) => S;
};
type UseFieldArrayReturn<T extends UnknownRecord, S extends ValidationSchema<T>> = {
  append: (values: T) => void;
  errors: Array<FormErrors<T>>;
  groups: Array<FieldGroup<T>>;
  handleReset: (update?: Update<Array<T>>) => void;
  handleSubmit: HandleSubmit$1<"FieldArray", T, S>;
  isSubmitting: boolean;
  remove: (index: number) => void;
  setErrors: SetFieldArrayErrors<T>;
  setValues: (index: number, update: Update<T>) => void;
  values: Array<T>;
};
declare function useFieldArray<T extends UnknownRecord, S extends ValidationSchema<T>>({
  initialValues,
  validationStrategy,
  validators,
}: UseFieldArrayConfig<T, S>): UseFieldArrayReturn<T, S>;

type UseFormConfig<T extends UnknownRecord, S extends ValidationSchema<T>> = {
  initialValues: T;
  validationStrategy: ValidationStrategy;
  validators: (values: T) => S;
};
type UseFormReturn<T extends UnknownRecord, S extends ValidationSchema<T>> = {
  errors: FormErrors<T>;
  fieldProps: <K extends keyof T>(key: K) => FieldProps<T[K]>;
  handleReset: (update?: Update<T>) => void;
  handleSubmit: HandleSubmit$1<"Form", T, S>;
  isSubmitting: boolean;
  setErrors: SetErrors<T>;
  setValues: (update: Update<T>) => void;
  values: T;
};
declare function useForm<T extends UnknownRecord, S extends ValidationSchema<T>>({
  initialValues,
  validationStrategy,
  validators,
}: UseFormConfig<T, S>): UseFormReturn<T, S>;

type RedundantFields = "handleSubmit" | "isSubmitting";
type CompositeValues<FormValues, FieldArrayValues> = {
  form: FormValues;
  fieldArray: FieldArrayValues;
};
type FieldArrayValidationSchema<FieldArrayValues extends UnknownFieldArray> = {
  [K in keyof FieldArrayValues]: ValidationSchema<FieldArrayValues[K][number]>;
};
type UseCompositeFormConfig<
  FormValues extends UnknownRecord,
  FormSchema extends ValidationSchema<FormValues>,
  FieldArrayValues extends UnknownFieldArray,
  FieldArraySchema extends FieldArrayValidationSchema<FieldArrayValues>,
> = {
  form: {
    initialValues: FormValues;
    validationStrategy: ValidationStrategy;
    validators: (values: CompositeValues<FormValues, FieldArrayValues>) => FormSchema;
  };
  fieldArray: {
    initialValues: FieldArrayValues;
    validationStrategy: ValidationStrategy;
    validators: (values: CompositeValues<FormValues, FieldArrayValues>) => FieldArraySchema;
  };
};
type FieldArrayReturn<FieldArrayValues extends UnknownFieldArray> = {
  [K in keyof FieldArrayValues]: Omit<UseFieldArrayReturn<FieldArrayValues[K][number], never>, RedundantFields>;
};
type FieldArrayValidatedValues<
  FieldArrayValues extends UnknownFieldArray,
  FieldArraySchema extends FieldArrayValidationSchema<FieldArrayValues>,
> = {
  [K in keyof FieldArrayValues]: Array<
    ValidatedValues<
      FieldArrayValues[K][number],
      FieldArraySchema[K] extends ValidationSchema<FieldArrayValues[K][number]>
        ? FieldArraySchema[K]
        : ValidationSchema<FieldArrayValues[K][number]>
    >
  >;
};
type OnSubmit<
  FormValues extends UnknownRecord,
  FormSchema extends ValidationSchema<FormValues>,
  FieldArrayValues extends UnknownFieldArray,
  FieldArraySchema extends FieldArrayValidationSchema<FieldArrayValues>,
> = (
  values: CompositeValues<
    ValidatedValues<FormValues, FormSchema>,
    FieldArrayValidatedValues<FieldArrayValues, FieldArraySchema>
  >,
) => Promise<unknown>;
type OnFailure<FormValues extends UnknownRecord, FieldArrayValues extends UnknownFieldArray> = (
  errors: CompositeValues<FormErrors<FormValues> | null, FieldArrayValidationFailure<FieldArrayValues> | null>,
) => unknown;
type OnSubmitMatch<
  FormValues extends UnknownRecord,
  FormSchema extends ValidationSchema<FormValues>,
  FieldArrayValues extends UnknownFieldArray,
  FieldArraySchema extends FieldArrayValidationSchema<FieldArrayValues>,
> = {
  onFailure: OnFailure<FormValues, FieldArrayValues>;
  onSuccess: OnSubmit<FormValues, FormSchema, FieldArrayValues, FieldArraySchema>;
};
type HandleSubmit<
  FormValues extends UnknownRecord,
  FormSchema extends ValidationSchema<FormValues>,
  FieldArrayValues extends UnknownFieldArray,
  FieldArraySchema extends FieldArrayValidationSchema<FieldArrayValues>,
> = {
  (onSubmit: OnSubmit<FormValues, FormSchema, FieldArrayValues, FieldArraySchema>): void;
  (onSubmit: OnSubmitMatch<FormValues, FormSchema, FieldArrayValues, FieldArraySchema>): void;
};
type UseCompositeFormReturn<
  FormValues extends UnknownRecord,
  FormSchema extends ValidationSchema<FormValues>,
  FieldArrayValues extends UnknownFieldArray,
  FieldArraySchema extends FieldArrayValidationSchema<FieldArrayValues>,
> = {
  fieldArray: FieldArrayReturn<FieldArrayValues>;
  form: Omit<UseFormReturn<FormValues, never>, RedundantFields>;
  handleReset: () => void;
  handleSubmit: HandleSubmit<FormValues, FormSchema, FieldArrayValues, FieldArraySchema>;
  isSubmitting: boolean;
};
declare function useCompositeForm<
  FormValues extends UnknownRecord,
  FormSchema extends ValidationSchema<FormValues>,
  FieldArrayValues extends UnknownFieldArray,
  FieldArraySchema extends FieldArrayValidationSchema<FieldArrayValues>,
>(
  config: UseCompositeFormConfig<FormValues, FormSchema, FieldArrayValues, FieldArraySchema>,
): UseCompositeFormReturn<FormValues, FormSchema, FieldArrayValues, FieldArraySchema>;

declare function fromZod<T, S extends ZodType>(zod: S): Validator<T, TypeOf<S>>;
type ZodSchema<T extends Record<string, unknown>> = {
  [K in keyof T]: ZodType | null;
};
type ToValidationSchema<T extends ZodSchema<Record<string, unknown>>> = {
  [K in keyof T]: T[K] extends ZodType ? Validator<input<T[K]>, output<T[K]>> : null;
};
declare function fromZodSchema<S extends ZodSchema<Record<string, unknown>>>(schema: S): ToValidationSchema<S>;

export {
  type Failure,
  type FieldArrayValidationFailure,
  type FieldGroup,
  type FieldProps,
  type FieldState,
  type FieldValidationConfig,
  type FormErrors,
  type FormState,
  type HandleSubmit$1 as HandleSubmit,
  type NonEmptyArray,
  type Predicate,
  type Refinement,
  type Result,
  type SetErrors,
  type SetFieldArrayErrors,
  type Success,
  type ToValidationSchema,
  type Toggle,
  type UnknownFieldArray,
  type UnknownRecord,
  type Update,
  type UseFieldArrayConfig,
  type UseFieldArrayReturn,
  type UseFormConfig,
  type UseFormReturn,
  type ValidatedValues,
  type ValidationSchema,
  type ValidationStrategy,
  type Validator,
  type ZodSchema,
  deleteAt,
  extract,
  failure$1 as failure,
  formStateManager,
  fromPredicate$1 as fromPredicate,
  fromZod,
  fromZodSchema,
  getErrors,
  getValues,
  initializeForm,
  isFailure,
  isNonEmpty,
  isOutOfBound,
  isSuccess,
  makeFieldGroups,
  makeFieldProps,
  map,
  modifyAt,
  of$1 as of,
  success$1 as success,
  updateValues,
  useCompositeForm,
  useFieldArray,
  useForm,
  validateCompositeFieldArray,
  validateFieldArray,
  validateForm,
  validator,
};
